﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace CarFire
{
    /// <summary>
    /// This class represents a projectile object that will be spawned whenever a player or a monster fires.
    /// </summary>
    public class Projectile
    {
        //Member Variables
        Vector2 velocity;
        Texture2D projectileModel;
        int damage;
        int gridX;
        int gridY;
        //The pixel position should be the pixel position on the map.  When a projectile is drawn
        //these will have to be transformed to the coordinate system that the drawable screen is using.
        int pixelX;
        int pixelY;
        MovementManager mMotion;
        Point mPosition;
        int mCharacterIndex;
        Game mGame;

        /// <summary>
        /// The Constructor for a projectile object.
        /// </summary>
        /// <param name="_currentMap">The map that this character will interact with</param>
        /// <param name="_projectileModel">The model for this projectile</param>
        /// <param name="_velocity">How fast the projectile moves</param>
        /// <param name="_gridX">The starting X position in the map grid</param>
        /// <param name="_gridY">The starting Y position in the map grid</param>
        /// <param name="_pixelX">The absolute X pixel position on the map</param>
        /// <param name="_pixelY"> The absolute Y pixel position on the map</param>
        public Projectile(Game theGame,
                            Texture2D _projectileModel,
                            Vector2 _velocity,
                            Point _position,
                            int characterNumber,
                            int _damage)
                
        {
            mGame = theGame;
            mCharacterIndex = characterNumber;
            projectileModel = _projectileModel;
            velocity = _velocity;
            mPosition = _position;
            damage = _damage;
            // Speed is the number of grid cells you can move through per second.
            mMotion = new MovementManager(mPosition, velocity.Length());
        }
        public void Update(TimeSpan timeSpan)
        {
            bool moveRight = false;
            bool moveLeft = false;
            bool moveUp = false;
            bool moveDown = false;
            if (velocity.X > 0)
                moveRight = true;
            else if (velocity.X < 0)
                moveLeft = true;
            if (velocity.Y > 0)
                moveDown = true;
            else if (velocity.Y < 0)
                moveUp = true;
            Point destination = MovementManager.GetNeighborCell(mMotion.Coordinates, moveLeft, moveRight, moveUp, moveDown);
            mMotion.Update(timeSpan, moveLeft, moveRight, moveUp, moveDown);
            
            
        }
        /// <summary>
        /// This method will draw a projectile to the screen
        /// </summary>
        /// <param name="spriteBatch"></param>
        /// <param name="topLeftX">The map X pixel position of the topLeft of the display</param>
        /// <param name="topLeftY">The map Y pixel position of the topLeft of the display</param>
        public void Draw(SpriteBatch spriteBatch)
        {
            Rectangle position = mGame.State.Map.GetRectangleFromCoordinates(mMotion.Position);
            spriteBatch.Draw(projectileModel, position, Color.White);
         
        }

        /// <summary>
        /// Basic getters and setters
        /// </summary>
        public int GridX { get { return gridX; } set { gridX = value; } }
        public int GridY { get { return gridY; } set { gridY = value; } }
        public int PixelX { get { return pixelX; } set { pixelX = value; } }
        public int PixelY { get { return pixelY; } set { pixelY = value; } }
        public int Damage { get { return damage; } set { damage = value; } }
        public Vector2 Position { get { return mMotion.Position; } }
        public Point Coordinates { get { return mMotion.Coordinates; } }
    }
}
