using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using CS_3505_Project_06.CS_3505;

namespace CS_3505_Project_06
{
    /// <summary>
    /// A game outline for testing network communications
    /// </summary>
    public class Game06 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        NetworkGame networkGame;

        ILobby lobby;
        IDeterministicGame deterministicGame;


        // Constructor

        public Game06()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            Components.Add(new GamerServicesComponent(this));

            lobby = new lobbyGUI();
            deterministicGame = new TestHarness();
            networkGame = new NetworkGame(lobby, deterministicGame);
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // Set a fixed time span of 1/60th of a second.
            IsFixedTimeStep = true;
            TargetElapsedTime = networkGame.TargetTimeSpan;

            // DEBUG: This is for the test harness.
            Mouse.SetPosition(400, 300);

            // Allow the base class to initialize.
            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.

            spriteBatch = new SpriteBatch(GraphicsDevice);

            lobby.LoadContent(Content, graphics);
            deterministicGame.LoadContent(Content);
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            lobby.UnloadContent();
            deterministicGame.UnloadContent();
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            networkGame.Update(gameTime);

            // Allow the superclass to do any needed updates (unknown purpose).
            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            // DEBUG: This is for the test harness.
            GraphicsDevice.Clear(new Color(16, 16, 16, 255));

            spriteBatch.Begin();

            // Let the game draw itself.
            networkGame.Draw(gameTime, spriteBatch);

            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
